import { push } from '../../../../services/route';
import { getAppInfoes, addApp, editApp, removeApp, deployApp, undeployApp, editDeployment } from '../../../../services/app';
import { getToken } from '../../../../utils/auth';

export default {
  namespace: 'appInfo',
  state: {
    allAppInfo: [],
    token: '',
    uploadURL: '',
    record: {},
  },
  subscriptions: {
    setup({ dispatch }) {  // eslint-disable-line
    },
  },
  effects: {
    *getTokens(action, { put, call }) {
      const token = yield call(getToken);
      yield put({ type: 'queryToken', payload: token });
    },
    *getAppInfo({ payload: { name } }, { put, call }) {
      const info = yield call(getAppInfoes, name);
      yield put({ type: 'queryAppInfo', payload: info });
    },
    *addAppInfo({ payload: { values } }, { call }) {
      yield call(addApp, values.name, values.packageName, values.description);
      push('../list');
    },
    *editAppInfo({ payload: { values } }, { call }) {
      yield call(editApp, values.name, values.newName, values.packageName, values.description);
      push('../list');
    },
    *delAppInfo({ payload: { name } }, { call, put }) {
      yield call(removeApp, name);
      yield put({ type: 'getAppInfo', payload: { } });
      push('../list');
    },
    *uploadAPK({ payload: { response } }, { put }) {
      if (response.errorCode !== 0) {
        throw new Error(response.message);
      }
      const uploadURL = response.data[0];
      yield put({ type: 'queryUploadURL', payload: uploadURL });
    },
    *addDeployApp({ payload: { values } }, { call, put }) {
      yield call(deployApp, values.idOrName, values.uri, values.description, values.release);
      const name = values.idOrName;
      yield put({ type: 'getAppInfo', payload: { name } });
    },
    *removeDeployApp({ payload: { id, name } }, { call, put }) {
      yield call(undeployApp, id);
      yield put({ type: 'getAppInfo', payload: { name } });
    },
    *editSaveDeploy({ payload: { values, name } }, { call, put }) {
      yield call(editDeployment, values.id, values.description, values.status);
      yield put({ type: 'getAppInfo', payload: { name } });
    },
  },
  reducers: {
    queryUploadURL(state, { payload: uploadURL }) {
      return {
        ...state,
        uploadURL,
      };
    },
    queryAppInfo(state, { payload: allAppInfo }) {
      return {
        ...state,
        allAppInfo,
      };
    },
    queryToken(state, { payload: token }) {
      return {
        ...state,
        token,
      };
    },
    saveRecord(state, { payload: record }) {
      return {
        ...state,
        record,
      };
    },
  },
};
